/* ovt_inverted.c --- 1/x transfer function for overmaker.c */
#include <stdio.h>
#include <stdlib.h>

#include "overmaker.h"
#include "overmakerdefs.h"

static char buf[BUFSZ];

/*
 * Sets up static and global symbols in the overdrive.c file generated
 * by overmaker.c
 */

void hdr_inverted () {
  commentln ("Constants for the inverted transfer function");
  codeln ("static float K;");
  codeln ("static float sqrK;");

#ifdef ADWS_PRE_DIFF
  commentln ("ovt_inverted : One sample memory");
  codeln ("static float adwZ;");
  commentln ("ovt_inverted : Positive feedback");
  codeln ("static float adwFb;");
#endif /* ADWS_PRE_DIFF */

#ifdef ADWS_POST_DIFF
  codeln ("static float adwZ1;");
#if 0
  codeln ("static float adwZ2;");
#endif
  codeln ("static float adwFb2;");
#endif /* ADWS_POST_DIFF */

#ifdef ADWS_GFB
  codeln ("static float adwGfb;");
  codeln ("static float adwGfZ = 0.0;");
#endif /* ADWS_GFB */

#ifdef ADWS_NO_LINEAR
  commentln ("ovt_inverted: non-linear control");
  codeln ("static float adwU = 1.0;");
#endif /* ADWS_NO_LINEAR */
}

void cfg_inverted () {
  commentln ("Computes the constants for transfer curve");
  codeln ("void cfg_inverted (float new_k) {");
  pushIndent ();
  codeln ("if (0.0 < new_k) {");
  pushIndent ();
  codeln ("K = new_k;");
  codeln ("sqrK = sqrt (K);");
  codeln ("printf (\"\\rGnK=%10.4f\", new_k);");
  codeln ("fflush (stdout);");
  popIndent ();
  codeln ("}");
  popIndent ();
  codeln ("}");
}

void ctl_inverted () {
  codeln ("void ctl_inverted (unsigned char uc) {");
  pushIndent ();
  codeln ("float u = (float) uc;");
  sprintf (buf, "float v = %g + ((%g - %g) * ((u * u) / 16129.0));",
	   ADWS_GAIN_LO,
	   ADWS_GAIN_HI,
	   ADWS_GAIN_LO);
  codeln (buf);
  codeln ("cfg_inverted (v);");
  popIndent ();
  codeln ("}");

#ifdef ADWS_PRE_DIFF
  vspace (3);
  commentln ("ovt_inverted:Sets the positive feedback");
  codeln ("void ctl_inverted_fb (unsigned char uc) {");
  pushIndent ();
  sprintf (buf, "adwFb = %g * (((float) uc) / 127.0);", ADWS_FB_MAX);
  codeln (buf);
  codeln ("printf (\"\\rFbk=%10.4f\", adwFb);");
  codeln ("fflush (stdout);");
  popIndent ();
  codeln ("}");
#endif /* ADWS_PRE_DIFF */

#ifdef ADWS_NO_LINEAR
  vspace (3);
  commentln ("ovt_inverted: Sets curve shape");
  codeln ("void ctl_inverted_v (unsigned char uc) {");
  pushIndent ();
  sprintf (buf, "adwU = %g + ((%g - %g) * (((float) uc) / 127.0));",
	   ADWS_V_LO,
	   ADWS_V_HI,
	   ADWS_V_LO);
  codeln (buf);
  codeln ("printf (\"\\rV  =%10.4f\", adwU);");
  codeln ("fflush (stdout);");
  popIndent ();
  codeln ("}");
#endif /* ADWS_NO_LINEAR */

#ifdef ADWS_POST_DIFF
  vspace (3);
  commentln ("ovt_inverted: Postdiff feedback control");
  codeln ("void ctl_inverted_fb2 (unsigned char uc) {");
  pushIndent ();
  sprintf (buf, "adwFb2 = %g * (((float) uc) / 127.0);", ADWS_FB2_MAX);
  codeln (buf);
  codeln ("printf (\"\\rFb2=%10.4f\", adwFb2);");
  codeln ("fflush (stdout);");
  popIndent ();
  codeln ("}");
#endif /* ADWS_POST_DIFF */

#ifdef ADWS_GFB
  vspace (3);
  commentln ("ovt_inverted: Global feedback control");
  codeln ("void ctl_inverted_gfb (unsigned char uc) {");
  pushIndent ();
  sprintf (buf, "adwGfb = %g * (((float) uc) / 127.0);", ADWS_GFB_MAX);
  codeln (buf);
  codeln ("printf (\"\\rGfb=%10.4f\", adwGfb);");
  codeln ("fflush (stdout);");
  popIndent ();
  codeln ("}");
#endif /* ADWS_GFB */
}

void ini_inverted () {
  sprintf (buf, "cfg_inverted (%g);", ADWS_GAIN);
  codeln (buf);
#ifdef ADWS_PRE_DIFF
  codeln ("adwFb = 0.6175;");
#endif /* ADWS_PRE_DIFF */
}

void xfr_inverted (char * xs, char * ys) {
  commentln ("Adaptive linear-non-linear transfer function");

#ifdef ADWS_GFB
  commentln ("Global negative feedback");
  sprintf (buf, "%s -= (adwGfb * adwGfZ);", xs);
  codeln (buf);
#endif /* ADWS_GFB */

#ifdef ADWS_PRE_DIFF
  codeln ("{");
  pushIndent ();
  sprintf (buf, "float temp = %s - adwZ;", xs);
  codeln (buf);
  sprintf (buf, "adwZ = %s + (adwZ * adwFb);", xs);
  codeln (buf);
  sprintf (buf, "%s = temp;", xs);
  codeln (buf);
  popIndent ();
  codeln ("}");
#endif /* ADWS_PRE_DIFF */

#ifdef ADWS_NO_LINEAR

  sprintf (buf, "if (%s < 0.0) {", xs);
  codeln (buf);
  pushIndent ();

#if 0
  sprintf (buf, "%s = (1.0 / (1.0 - (sqrK * %s))) - 1.0;", ys, xs);
#else
  sprintf (buf,
	   "%s = (1.0/(1.0+(sqrK * %s * ((adwU * %s)+(adwU - 1.0))))) - 1.0;",
	   ys, xs, xs);
#endif

  codeln (buf);

  popIndent ();
  codeln ("} else {");
  pushIndent ();


#if 0
  sprintf (buf, "%s = 1.0 - (1.0 / (1.0 + (sqrK * %s)));", ys, xs);
#else
  sprintf (buf,
	   "%s = 1.0-(1.0/(1.0+(sqrK * %s * ((adwU * %s) + (1.0 - adwU)))));",
	   ys, xs, xs);
#endif

  codeln (buf);

  popIndent ();
  codeln ("}");
#else
  /* CODE REMOVED */
#endif /* not ADWS_NO_LINEAR */

#ifdef ADWS_POST_DIFF
  codeln ("{");
  pushIndent ();
#if 0
  sprintf (buf, "float temp = %s - adwZ1 + (adwFb2 * adwZ2);", ys);
  codeln (buf);
  sprintf (buf, "adwZ1 = %s;", ys);
  codeln (buf);
  sprintf (buf, "%s = adwZ2 = temp;", ys);
  codeln (buf);
#else
  /* This should be equivalent to the above, saving one delay element */
  sprintf (buf, "float temp = %s + (adwFb2 * adwZ1);", ys);
  codeln (buf);
  sprintf (buf, "%s = temp - adwZ1;", ys);
  codeln (buf);
  codeln ("adwZ1 = temp;");
#endif
  popIndent ();
  codeln ("}");
#endif /* ADWS_POST_DIFF */
  
#ifdef ADWS_GFB
  commentln ("Global negative feedback");
  sprintf (buf, "adwGfZ = %s;", ys);
  codeln (buf);
#endif /* ADWS_GFB */
}
